@@ -14,6 +14,7 @@
 ##' @param tol Convergence tolerance for EM algorithm. Default is 0.0001.
 ##' @param print.para Logical; if \code{TRUE}, prints parameter values at each iteration.
 ##' @param initial.para Optional list of initialized parameters. Default is \code{NULL}.
+##' @param cpu.cores Number of CPU cores for parallel computation. Default is 1. 
 ##'
 ##' @return  Object of class \code{mvjmcs} with elements
 ##' \item{beta}{the vector of all biomarker-specific fixed effects for the linear mixed effects sub-models.} 
@@ -34,6 +35,8 @@
 ##' \item{convergence}{convergence identifier: 1 corresponds to successful convergence, 
 ##' whereas 0 to a problem (i.e., when 0, usually more iterations are required).}
 ##' \item{vcov}{the variance-covariance matrix of all the fixed effects for both models.}
+##' \item{FisherInfo}{the Empirical Fisher information matrix.}
+##' \item{Score}{a matrix of the score function for all subjects.}
 ##' \item{sebeta}{the standard error of \code{beta}.}
 ##' \item{segamma1}{the standard error of \code{gamma1}.}
 ##' \item{segamma2}{the standard error of \code{gamma2}. 
@@ -63,6 +66,8 @@
 ##' 
 ##'   require(FastJM)
 ##'   require(survival)
+##'   require(future)
+##'   require(future.apply)
 ##'   
 ##'   data(mvcdata)
 ##'   data(mvydata)
@@ -94,10 +99,11 @@ mvjmcs <- function(ydata, cdata, long.formula,
                    random = NULL, surv.formula,
                    maxiter = 10000, opt = "nlminb", tol = 0.005,
                    print.para = TRUE, 
-                   initial.para = NULL){
+                   initial.para = NULL, cpu.cores = NULL){
   
   start_time <- Sys.time()
   
+  # ---- Longitudinal setup ----
   if(is.list(long.formula)){
     numBio = length(long.formula)
   } else {
@@ -106,8 +112,6 @@ mvjmcs <- function(ydata, cdata, long.formula,
   
   random.form <- RE  <- model <- vector("list", numBio)
   
-  # <- ID
-  
   if(is.list(random)){
     for(g in 1:numBio){
       random.form[[g]] <- all.vars(random[[g]])
@@ -118,20 +122,18 @@ mvjmcs <- function(ydata, cdata, long.formula,
         RE[[g]] <- random.form[[g]][-length(all.vars(random[[g]]))]
         model[[g]] <- "interslope"
       }
-      # ID[[g]] <- random.form[[g]][length(all.vars(random[[g]]))]
     }
     ID <- random.form[[g]][length(all.vars(random[[g]]))]
   } else {
     for(g in 1:numBio){
       random.form[[g]] <- all.vars(random)
-      if(length(random) == 1){
+      if(!is.list(random)){
         # RE[[g]] <- NULL
         model[[g]] <- "intercept"
       } else {
         RE[[g]] <- random.form[[g]][-length(all.vars(random))]
         model[[g]] <- "interslope"
       }
-      # ID[[g]] <- random.form[[g]][length(all.vars(random))]
     }
     ID <- random.form[[g]][length(all.vars(random))]
   }
@@ -139,7 +141,7 @@ mvjmcs <- function(ydata, cdata, long.formula,
   lengthb <- length(long.formula)
   long <- list()
   
-  # check for every biomarker
+  # ---- Formula checks for each biomarker ----
   for(g in 1:lengthb){
     if (!inherits(long.formula[[g]], "formula") || length(long.formula[[g]]) != 3) {
       stop("\nLinear mixed effects model must be a formula of the form \"resp ~ pred\"")
@@ -151,9 +153,12 @@ mvjmcs <- function(ydata, cdata, long.formula,
     longfmla[g] <- long.formula[g]
   }
   
-  # survival <- all.vars(surv.formula)
-  # random.form <- all.vars(random)
-  # ID <- random.form[length(random.form)]
+  
+  # ---- CPU setup ----
+  if(is.null(cpu.cores)){
+    cpu.cores <- 1
+  }
+
   cnames <- colnames(cdata)
   ynames <- colnames(ydata)
   
@@ -171,8 +176,6 @@ mvjmcs <- function(ydata, cdata, long.formula,
     stop("Numerical failure occurred when fitting a linear mixed effects model for initial guess.")
   }
   
-  # need to rearrange this part
-  numBio = length(long.formula)
   
   mdataM <- mdataSM <- vector("list", numBio)
   
@@ -225,26 +228,16 @@ mvjmcs <- function(ydata, cdata, long.formula,
   
   iter=0
   
+  # ======================== #
+  # ~~~~ Competing Risk ~~~~ #
+  # ======================== #
   if(CompetingRisk == TRUE){
     
-    CUH01 <- rep(0, n)
-    CUH02 <- rep(0, n)
-    HAZ01 <- rep(0, n)
-    HAZ02 <- rep(0, n)
-    
-    CumuH01 <- cumsum(H01[, 3])
-    CumuH02 <- cumsum(H02[, 3])
-    
-    getHazard(CumuH01, CumuH02, getinit$survtime, getinit$cmprsk, H01, H02, CUH01, CUH02, HAZ01, HAZ02)
-    
-    HAZ0 <- list(HAZ01, HAZ02)
-    
     pREvec <- c()
     
     for(g in 1:numBio){
       pREvec[g] <- ncol(getinit$Z[[g]])
     }
-    
     pREtotal <- sum(pREvec)
     
     index = 0
@@ -261,21 +254,12 @@ mvjmcs <- function(ydata, cdata, long.formula,
       Sig <- getinit$Sig
     }
     
-    data <- list(beta = getinit$beta, gamma1 = getinit$gamma1, gamma2 = getinit$gamma2,
-                 alpha = getinit$alpha, sigma = getinit$sigma,
-                 Z = getinit$Z, X1 = getinit$X1, Y = getinit$Y, Sig = Sig,
-                 CUH01 = CUH01, CUH02 = CUH02, HAZ01 = HAZ01, HAZ02 = HAZ02,
-                 mdataM = mdataM, mdataSM = mdataSM,
-                 cmprsk = getinit$cmprsk, W = getinit$W)
     
     numSubj <- n
-    numBio <- length(data$X1)
-    opt <- list()
+    # opt <- list()
     pos.mode <- vector("list", numSubj)
     subX1 <- subY <- subZ <- vector("list", numSubj)
     pos.cov <- list()
-    submdataM <- vector("list", numBio)
-    submdataSM <- vector("list", numBio)
     namesbeta <- vector("list", numBio)
     pbeta <- c()
     for (g in 1:numBio) {
@@ -284,120 +268,62 @@ mvjmcs <- function(ydata, cdata, long.formula,
     }
     namesgamma <- names(getinit$gamma1)
     
-    for(j in 1:numSubj) {
+    beta <- unlist(getinit$beta)
+    betaList <- getinit$beta
+    gamma1 <- getinit$gamma1
+    gamma2 <- getinit$gamma2
+    alphaList <- getinit$alpha
+    alpha1 <- unlist(alphaList[[1]])
+    alpha2 <- unlist(alphaList[[2]])
+    sigma <- getinit$sigma
+    
+    for(j in 1:numSubj){
       subX1[[j]] <- vector("list", numBio)
       for (g in 1:numBio) {
         
-        numRep <- data$mdataM[[g]][j]
-        indexStart <- data$mdataSM[[g]][j]
+        numRep <- mdataM[[g]][j]
+        indexStart <- mdataSM[[g]][j]
         
-        subX1[[j]][[g]] <- data$X1[[g]][indexStart:(indexStart+numRep-1),, drop = FALSE]
-        subY[[j]][[g]] <- data$Y[[g]][indexStart:(indexStart+numRep-1)]
-        subZ[[j]][[g]] <- data$Z[[g]][indexStart:(indexStart+numRep-1),, drop = FALSE]
+        subX1[[j]][[g]] <- getinit$X1[[g]][indexStart:(indexStart+numRep-1),, drop = FALSE]
+        subY[[j]][[g]] <- getinit$Y[[g]][indexStart:(indexStart+numRep-1)]
+        subZ[[j]][[g]] <- getinit$Z[[g]][indexStart:(indexStart+numRep-1),, drop = FALSE]
       }
-      
-      subCUH01 <- data$CUH01[j]
-      subCUH02 <- data$CUH02[j]
-      subHAZ01 <- data$HAZ01[j]
-      subHAZ02 <- data$HAZ02[j]
-      subcmprsk <- data$cmprsk[j]
-      subW <- t(data$W[j, ])
-      
-      subdata <- list(
-        beta = data$beta,
-        gamma1 = data$gamma1,
-        gamma2 = data$gamma2,
-        alpha = data$alpha,
-        sigma = data$sigma,
-        Z = subZ[[j]],
-        X1 = subX1[[j]],
-        Y = subY[[j]],
-        Sig = data$Sig,
-        CUH01 = subCUH01,
-        CUH02 = subCUH02,
-        HAZ01 = subHAZ01,
-        HAZ02 = subHAZ02,
-        mdataM = submdataM,
-        mdataSM = submdataSM,
-        cmprsk = subcmprsk,
-        W = subW
-      )
-      
-      opt <- optim(
-        par = c(rep(0, pREtotal)), # CHANGE THIS PART
-        getbSig,
-        getbSig_grad,
-        data = subdata,
-        method = "BFGS",
-        hessian = TRUE
-      )
-      pos.mode[[j]] <- opt$par
-      pos.cov[[j]] <- solve(opt$hessian)
-      
     }
     
     survtime <- getinit$survtime
     cmprsk <- getinit$cmprsk
+    W <- getinit$W
+    ngamma <- ncol(W)
     
-    output <- normalApprox(subX1,subY, subZ, getinit$W,
-                           mdataM, mdataSM,
-                           pos.mode,  getinit$sigma, pos.cov,
-                           H01, H02, getinit$survtime, getinit$cmprsk,
-                           getinit$gamma1, getinit$gamma2, getinit$alpha,
-                           CUH01, CUH02,HAZ01,HAZ02, Sig, subdata$beta)
-    
-    # PAR UPDATE HERE
-    
-    tempbeta <- output$beta
-    # tempsigma <- c(output$sigma1, output$sigma2)
-    tempphi1 <- output$phi1
-    tempphi2 <- output$phi2
-    
-    it <- 1
-    tempSig <- list()
-    tempSig[[it]] <- output$Sig
-    ngamma <- ncol(data$W)
-    index = 0
-    gamma1 <- output$phi1[(index+1):ngamma]
-    gamma2 <- output$phi2[(index+1):ngamma]
-    
-    # gamma1 hiim<- c(1,0.5)
-    # gamma2 <- c(-0.5,0.5)
-    index = index + ngamma
-    #NEED TO ADJUST THIS PART
-    alpha1 <- output$phi1[(ngamma+1):(length(output$phi1))] # 3 and 2 come from competing risk
-    alpha2 <- output$phi2[(ngamma+1):(length(output$phi2))]
-    
-    alpha1g <- alpha2g <- vector("list", numBio)
-    index = 0
-    for(g in 1:numBio){
-      alpha1g[[g]] <- alpha1[(index+1):(index + pREvec[g])]
-      alpha2g[[g]] <- alpha2[(index+1):(index + pREvec[g])]
-      index = index + pREvec[g]
-    }
-    
-    
-    alphaList <- list(alpha1g, alpha2g)
-    
-    iter=0
-    beta <- output$beta
-    sigma <- output$sigmaVec
-    Sig <- output$Sig
+    old_parallelly <- options(
+      parallelly.makeNodePSOCK.rscript_args = c("--vanilla")
+    )
+    on.exit(options(old_parallelly), add = TRUE)
+    old_plan <- future::plan()
+    future::plan(future::multisession, workers = cpu.cores)
+    on.exit(future::plan(old_plan), add = TRUE)
     
     repeat{
-      
       iter <- iter + 1
+      
+      CUH01 <- rep(0, n) 
+      CUH02 <- rep(0, n) 
+      HAZ01 <- rep(0, n) 
+      HAZ02 <- rep(0, n) 
+      CumuH01 <- cumsum(H01[, 3]) 
+      CumuH02 <- cumsum(H02[, 3]) 
+      getHazard(CumuH01, CumuH02, survtime, cmprsk, H01, H02, CUH01, CUH02, HAZ01, HAZ02) 
+      
       prebeta <- beta
       pregamma1 <- gamma1
       pregamma2 <- gamma2
-      
       prealphaList <- alphaList
-      prealpha1 <- alpha1 # 3 and 2 come from competing risk
+      prealpha1 <- alpha1
       prealpha2 <- alpha2
       preH01 <- H01
       preH02 <- H02
       preSig <- Sig
-      presigma <- sigma # for checking
+      presigma <- sigma
       
       if (print.para) {
         writeLines("iter is:")
@@ -418,95 +344,36 @@ mvjmcs <- function(ydata, cdata, long.formula,
         print(prealpha2)
       }
       
-      CUH01 <- rep(0, n)
-      CUH02 <- rep(0, n)
-      HAZ01 <- rep(0, n)
-      HAZ02 <- rep(0, n)
+      data <- list(
+        beta = betaList, gamma1 = gamma1, gamma2 = gamma2,
+        alpha = alphaList, sigma = sigma,
+        Z = subZ, X1 = subX1, Y = subY, Sig = Sig,
+        CUH01 = CUH01, CUH02 = CUH02, HAZ01 = HAZ01, HAZ02 = HAZ02,
+        cmprsk = cmprsk, W = W
+      )
       
-      CumuH01 <- cumsum(output$H01[, 3])
-      CumuH02 <- cumsum(output$H02[, 3])
+      res <- future.apply::future_lapply(seq_len(numSubj), estepMV_worker, future.seed = TRUE, future.scheduling = 2,
+                                         data, pREtotal)
+      pos.mode <- lapply(res, `[[`, "mode")
+      pos.cov  <- lapply(res, function(x) crossprod(x$ccov))
       
-      getHazard(CumuH01, CumuH02, survtime, cmprsk, preH01, preH02, CUH01, CUH02, HAZ01, HAZ02)
+      output <- normalApprox(
+        subX1, subY, subZ, W,
+        mdataM, mdataSM,
+        pos.mode, sigma, pos.cov,
+        H01, H02, survtime, cmprsk,
+        gamma1, gamma2, alphaList,
+        CUH01, CUH02, HAZ01, HAZ02, Sig, betaList
+      )
       
-      data <- list(beta = prebeta, gamma1 = pregamma1, gamma2 = pregamma2,
-                   alpha = prealphaList, sigma = presigma,
-                   Z = getinit$Z, X1 = getinit$X1, Y = getinit$Y, Sig = preSig,
-                   CUH01 = CUH01, CUH02 = CUH02, HAZ01 = HAZ01, HAZ02 = HAZ02,
-                   mdataM = mdataM, mdataSM = mdataSM,
-                   cmprsk = getinit$cmprsk, W = getinit$W)
-      
-      for(j in 1:numSubj) {
-        
-        subCUH01 <- CUH01[j]
-        subCUH02 <- CUH02[j]
-        subHAZ01 <- HAZ01[j]
-        subHAZ02 <- HAZ02[j]
-        subcmprsk <- data$cmprsk[j]
-        subW <- t(data$W[j, ])
-        
-        subdata <- list(
-          beta = output$betaList,
-          gamma1 = data$gamma1,
-          gamma2 = data$gamma2,
-          alpha = data$alpha,
-          sigma = data$sigma,
-          Z = subZ[[j]],
-          X1 = subX1[[j]],
-          Y = subY[[j]],
-          Sig = data$Sig,
-          CUH01 = subCUH01,
-          CUH02 = subCUH02,
-          HAZ01 = subHAZ01,
-          HAZ02 = subHAZ02,
-          mdataM = submdataM,
-          mdataSM = submdataSM,
-          cmprsk = subcmprsk,
-          W = subW
-        )
-        
-        opt <- optim(
-          par = rep(0, pREtotal),
-          getbSig,
-          getbSig_grad,
-          data = subdata,
-          method = "BFGS",
-          hessian = TRUE
-        )
-        
-        pos.mode[[j]] <- opt$par
-        pos.cov[[j]] <- solve(opt$hessian)
-      }
-      
-      
-      # if(method == "quad"){
-      #   output <- getQuadMix(subX1,subY, subZ, getinit$W,
-      #                        mdataM, mdataSM,
-      #                        pos.mode, presigma, pos.cov, weight.c, abscissas.c,
-      #                        H01, H02, getinit$survtime, getinit$cmprsk,
-      #                        data$gamma1, data$gamma2, data$alpha,
-      #                        CUH01, CUH02,HAZ01,HAZ02,preSig, subdata$beta)
-      #   
-      # } else {
-      output <- normalApprox(subX1,subY, subZ, getinit$W,
-                             mdataM, mdataSM,
-                             pos.mode, presigma, pos.cov,
-                             H01, H02, getinit$survtime, getinit$cmprsk,
-                             data$gamma1, data$gamma2, data$alpha,
-                             CUH01, CUH02,HAZ01,HAZ02,preSig, subdata$beta)
-      # }
-      
-      # PAR UPDATE HERE - for debugging purposes
       beta <- output$beta
+      betaList <- output$betaList
       sigma <- output$sigmaVec
       Sig <- output$Sig
       
-      # tempsigma <- rbind(tempsigma, sigma)
-      # CHANGE THIS PART
       gamma1 <- output$phi1[1:ngamma]
       gamma2 <- output$phi2[1:ngamma]
-      # gamma1 <- c(1,0.5)
-      # gamma2 <- c(-0.5,0.5)
-      alpha1 <- output$phi1[(ngamma+1):(length(output$phi1))] # 3 and 2 come from competing risk
+      alpha1 <- output$phi1[(ngamma+1):(length(output$phi1))]
       alpha2 <- output$phi2[(ngamma+1):(length(output$phi2))]
       alpha1g <- alpha2g <- vector("list", numBio)
       index = 0
@@ -517,14 +384,12 @@ mvjmcs <- function(ydata, cdata, long.formula,
       }
       
       alphaList <- list(alpha1g, alpha2g)
-      
-      it <- it + 1
-      tempSig[[it]] <- Sig
       H01 <- output$H01
       H02 <- output$H02
       
-      # leave condition
-      if((mvDiff(beta, prebeta, sigma, presigma, gamma1, pregamma1, gamma2, pregamma2,
+      
+      
+      if((mvDiffrelative(beta, prebeta, sigma, presigma, gamma1, pregamma1, gamma2, pregamma2,
                  alpha1, prealpha1, alpha2, prealpha2,
                  Sig, preSig, H01, preH01, H02, preH02, tol) == 0) || (iter == maxiter) #|| (!is.list(GetEfun)) || (!is.list(GetMpara))
       ) {
@@ -536,16 +401,38 @@ mvjmcs <- function(ydata, cdata, long.formula,
     if (iter == maxiter) {
       writeLines("program stops because of nonconvergence")
       convergence = 0
-      sebeta <- sesigma <- segamma1 <- segamma2 <- sealpha1 <- sealpha2 <- seSig <- NULL
+      sebeta <- sesigma <- segamma1 <- segamma2 <- sealpha1 <- sealpha2 <- seSig <- vcov <- NULL
       
     } else {
       convergence = 1
-      SEest <- getmvCov(beta, gamma1, gamma2, 
-                        alpha1, alpha2, 
-                        H01, H02, pos.cov, Sig, sigma, 
-                        subX1, subY, subZ, getinit$W, 
-                        getinit$survtime,getinit$cmprsk,
-                        mdataM, mdataSM, pos.mode)
+      ## run 1 estep here to get posterior mean and covariance --> use estimate to plug in to line47
+      
+      CUH01 <- rep(0, n)
+      CUH02 <- rep(0, n)
+      HAZ01 <- rep(0, n)
+      HAZ02 <- rep(0, n)
+      CumuH01 <- cumsum(H01[, 3])
+      CumuH02 <- cumsum(H02[, 3])
+      getHazard(CumuH01, CumuH02, survtime, cmprsk, H01, H02, CUH01, CUH02, HAZ01, HAZ02)
+      
+      data <- list(beta = betaList, gamma1 = gamma1, gamma2 = gamma2,
+                   alpha = alphaList, sigma = sigma,
+                   Z = subZ, X1 = subX1, Y = subY, Sig = Sig,
+                   CUH01 = CUH01,  HAZ01 = HAZ01, CUH02 = CUH02,  HAZ02 = HAZ02,
+                   mdataM = mdataM, mdataSM = mdataSM,
+                   cmprsk = cmprsk, W = W)
+      
+      res <- future.apply::future_lapply(seq_len(numSubj), estepMV_worker, future.seed = TRUE, future.scheduling = 2,
+                                         data, pREtotal)
+      pos.mode <- lapply(res, `[[`, "mode")
+      pos.cov  <- lapply(res, function(x) crossprod(x$ccov))
+      
+      SEest <- getmvCov(beta, gamma1, gamma2,
+                          alpha1, alpha2,
+                          H01, H02, pos.cov, Sig, sigma, 
+                          subX1, subY, subZ, getinit$W, 
+                          getinit$survtime,getinit$cmprsk,
+                          mdataM, mdataSM, pos.mode)
       
       sebeta <- SEest$sebeta
       sesigma <- SEest$sesigma
@@ -555,11 +442,14 @@ mvjmcs <- function(ydata, cdata, long.formula,
       sealpha2 <- SEest$sealpha2
       seSig <- SEest$seSig
       vcov <- SEest$vcov
-      
+      FisherInfo <- SEest$FisherInfo
+      Score <- SEest$Score
       
     }
     
     end_time <- Sys.time()
+    
+    writeLines("runtime is:")
     print(runtime <- end_time - start_time)
     
     PropComp <- as.data.frame(table(cdata[, survival[2]]))
@@ -575,15 +465,12 @@ mvjmcs <- function(ydata, cdata, long.formula,
       betacount <- betacount + pbeta[g]
     }
     
-    # return(list(output = output, re = pos.mode, sigi = pos.cov, 
-    #             beta = beta, sigmaout = sigma, gamma1 = gamma1, gamma2 = gamma2, alpha1 = alpha1, alpha2 = alpha2,
-    #             SEest = SEest, runtime = runtime, iter = iter))
     names(beta) <- unlist(namesbeta)
     
-    result <- list(beta = beta, betaList = betaList, gamma1 = gamma1, gamma2 = gamma2, 
+    result <- list(beta = beta, betaList = output$betaList, gamma1 = gamma1, gamma2 = gamma2, 
                    alpha1 = alpha1, alpha2 = alpha2, H01 = H01, H02 = H02, 
                    Sig = Sig, sigma = sigma, iter = iter, convergence = convergence, 
-                   vcov = vcov, sebeta = sebeta, segamma1 = segamma1, segamma2 = segamma2, 
+                   vcov = vcov, FisherInfo = FisherInfo, Score = Score, sebeta = sebeta, segamma1 = segamma1, segamma2 = segamma2, 
                    sealpha1 = sealpha1, sealpha2 = sealpha2, seSig = seSig, sesigma = sesigma, pos.mode = pos.mode, pos.cov = pos.cov,
                    CompetingRisk = CompetingRisk, ydata = rawydata, cdata = rawcdata, 
                    PropEventType = PropComp, LongitudinalSubmodel = long.formula,
@@ -596,17 +483,9 @@ mvjmcs <- function(ydata, cdata, long.formula,
     return(result)
     
   } else {
-    # ~~~~~~~~~~~~~
-    # SINGLE FAILURE
-    # ~~~~~~~~~~~~~
-    
-    CUH01 <- rep(0, n)
-    HAZ01 <- rep(0, n)
-    CumuH01 <- cumsum(H01[, 3])
-    
-    getHazardSF(CumuH01, getinit$survtime, getinit$cmprsk, H01,  CUH01, HAZ01)
-    
-    HAZ0 <- list(HAZ01)
+    # ======================== #
+    # ~~~~ Single Failure ~~~~ #
+    # ======================== #
     
     pREvec <- c()
     
@@ -629,21 +508,11 @@ mvjmcs <- function(ydata, cdata, long.formula,
       Sig <- getinit$Sig
     }
     
-    data <- list(beta = getinit$beta, gamma1 = getinit$gamma1,
-                 alpha = getinit$alpha, sigma = getinit$sigma,
-                 Z = getinit$Z, X1 = getinit$X1, Y = getinit$Y, Sig = Sig,
-                 CUH01 = CUH01, HAZ01 = HAZ01, 
-                 mdataM = mdataM, mdataSM = mdataSM,
-                 cmprsk = getinit$cmprsk, W = getinit$W)
-    
     numSubj <- n
-    numBio <- length(data$X1)
-    opt <- list()
+    # opt <- list()
     pos.mode <- vector("list", numSubj)
     subX1 <- subY <- subZ <- vector("list", numSubj)
     pos.cov <- list()
-    submdataM <- vector("list", numBio)
-    submdataSM <- vector("list", numBio)
     namesbeta <- vector("list", numBio)
     pbeta <- c()
     for (g in 1:numBio) {
@@ -651,105 +520,55 @@ mvjmcs <- function(ydata, cdata, long.formula,
       pbeta[g] <- length(getinit$beta[[g]])
     }
     namesgamma <- names(getinit$gamma1)
+    beta <- unlist(getinit$beta)
+    betaList <- getinit$beta
+    gamma1 <- getinit$gamma1
+    alphaList <- getinit$alpha
+    alpha1 <- unlist(alphaList[[1]])
+    sigma <- getinit$sigma
     
-    for(j in 1:numSubj) {
+    for(j in 1:numSubj){
       subX1[[j]] <- vector("list", numBio)
       for (g in 1:numBio) {
         
-        numRep <- data$mdataM[[g]][j]
-        indexStart <- data$mdataSM[[g]][j]
+        numRep <- mdataM[[g]][j]
+        indexStart <- mdataSM[[g]][j]
         
-        subX1[[j]][[g]] <- data$X1[[g]][indexStart:(indexStart+numRep-1),, drop = FALSE]
-        subY[[j]][[g]] <- data$Y[[g]][indexStart:(indexStart+numRep-1)]
-        subZ[[j]][[g]] <- data$Z[[g]][indexStart:(indexStart+numRep-1),, drop = FALSE]
+        subX1[[j]][[g]] <- getinit$X1[[g]][indexStart:(indexStart+numRep-1),, drop = FALSE]
+        subY[[j]][[g]] <- getinit$Y[[g]][indexStart:(indexStart+numRep-1)]
+        subZ[[j]][[g]] <- getinit$Z[[g]][indexStart:(indexStart+numRep-1),, drop = FALSE]
       }
-      
-      subCUH01 <- data$CUH01[j]
-      subHAZ01 <- data$HAZ01[j]
-      subcmprsk <- data$cmprsk[j]
-      subW <- t(data$W[j, ])
-      
-      subdata <- list(
-        beta = data$beta,
-        gamma1 = data$gamma1,
-        alpha = getinit$alpha,
-        sigma = data$sigma,
-        Z = subZ[[j]],
-        X1 = subX1[[j]],
-        Y = subY[[j]],
-        Sig = data$Sig,
-        CUH01 = subCUH01,
-        HAZ01 = subHAZ01,
-        mdataM = submdataM,
-        mdataSM = submdataSM,
-        cmprsk = subcmprsk,
-        W = subW
-      )
-      
-      opt <- optim(
-        par = c(rep(0, pREtotal)), # CHANGE THIS PART
-        getbSigSF,
-        getbSig_gradSF,
-        data = subdata,
-        method = "BFGS",
-        hessian = TRUE
-      )
-      pos.mode[[j]] <- opt$par
-      pos.cov[[j]] <- solve(opt$hessian)
-      
     }
     
     survtime <- getinit$survtime
     cmprsk <- getinit$cmprsk
+    W <- getinit$W
+    ngamma <- ncol(W)
     
-    output <- normalApproxSF(subX1,subY, subZ, getinit$W,
-                             mdataM, mdataSM,
-                             pos.mode,  getinit$sigma, pos.cov, 
-                             H01, getinit$survtime, getinit$cmprsk,
-                             getinit$gamma1, getinit$alpha,
-                             CUH01,HAZ01,Sig, subdata$beta)
-    
-    tempbeta <- output$beta
-    tempphi1 <- output$phi1
-    
-    it <- 1
-    tempSig <- list()
-    tempSig[[it]] <- output$Sig
-    ngamma <- ncol(data$W)
-    index = 0
-    gamma1 <- output$phi1[(index+1):ngamma]
-    
-    # gamma1 hiim<- c(1,0.5)
-    # gamma2 <- c(-0.5,0.5)
-    index = index + ngamma
-    #NEED TO ADJUST THIS PART
-    alpha1 <- output$phi1[(ngamma+1):(length(output$phi1))] # 3 and 2 come from competing risk
-    
-    alpha1g <- vector("list", numBio)
-    index = 0
-    for(g in 1:numBio){
-      alpha1g[[g]] <- alpha1[(index+1):(index + pREvec[g])]
-      index = index + pREvec[g]
-    }
-    
-    
-    alphaList <- list(alpha1g)
-    
-    iter=0
-    beta <- output$beta
-    sigma <- output$sigmaVec
-    Sig <- output$Sig
+    old_parallelly <- options(
+      parallelly.makeNodePSOCK.rscript_args = c("--vanilla")
+    )
+    on.exit(options(old_parallelly), add = TRUE)
+    old_plan <- future::plan()
+    future::plan(future::multisession, workers = cpu.cores)
+    on.exit(future::plan(old_plan), add = TRUE)
     
     repeat{
       
       iter <- iter + 1
+      
+      CUH01 <- rep(0, n) 
+      HAZ01 <- rep(0, n) 
+      CumuH01 <- cumsum(H01[, 3]) 
+      getHazardSF(CumuH01,  survtime, cmprsk, H01, CUH01, HAZ01) 
+      
       prebeta <- beta
       pregamma1 <- gamma1
       prealphaList <- alphaList
-      prealpha1 <- alpha1
+      prealpha1 <- alpha1 
       preH01 <- H01
       preSig <- Sig
-      presigma <- sigma # for checking
+      presigma <- sigma
       
       if (print.para) {
         writeLines("iter is:")
@@ -766,74 +585,33 @@ mvjmcs <- function(ydata, cdata, long.formula,
         print(prealpha1)
       }
       
-      CUH01 <- rep(0, n)
-      HAZ01 <- rep(0, n)
+      data <- list(beta = betaList, gamma1 = gamma1,
+                   alpha = alphaList, sigma = sigma,
+                   Z = subZ, X1 = subX1, Y = subY, Sig = Sig,
+                   CUH01 = CUH01, HAZ01 = HAZ01, 
+                   cmprsk = cmprsk, W = W)
       
-      CumuH01 <- cumsum(output$H01[, 3])
+      # specify cpu amounts
       
-      getHazardSF(CumuH01, survtime, cmprsk, preH01, CUH01, HAZ01)
+      res <- future.apply::future_lapply(seq_len(numSubj), estepMV_workerSF, future.seed = TRUE, future.scheduling = 2,
+                                         data, pREtotal)
+      pos.mode <- lapply(res, `[[`, "mode")
+      pos.cov <- lapply(res, function(x) crossprod(x$ccov)) 
       
-      data <- list(beta = prebeta, gamma1 = pregamma1,
-                   alpha = prealphaList, sigma = presigma,
-                   Z = getinit$Z, X1 = getinit$X1, Y = getinit$Y, Sig = preSig,
-                   CUH01 = CUH01, HAZ01 = HAZ01,
-                   mdataM = mdataM, mdataSM = mdataSM,
-                   cmprsk = getinit$cmprsk, W = getinit$W)
-      
-      for(j in 1:numSubj) {
-        
-        subCUH01 <- CUH01[j]
-        subHAZ01 <- HAZ01[j]
-        subcmprsk <- data$cmprsk[j]
-        subW <- t(data$W[j, ])
-        
-        subdata <- list(
-          beta = output$betaList,
-          gamma1 = data$gamma1,
-          alpha = data$alpha,
-          sigma = data$sigma,
-          Z = subZ[[j]],
-          X1 = subX1[[j]],
-          Y = subY[[j]],
-          Sig = data$Sig,
-          CUH01 = subCUH01,
-          HAZ01 = subHAZ01,
-          mdataM = submdataM,
-          mdataSM = submdataSM,
-          cmprsk = subcmprsk,
-          W = subW
-        )
-        
-        opt <- optim(
-          par = rep(0, pREtotal),
-          getbSigSF,
-          getbSig_gradSF,
-          data = subdata,
-          method = "BFGS",
-          hessian = TRUE
-        )
-        
-        pos.mode[[j]] <- opt$par
-        pos.cov[[j]] <- solve(opt$hessian)
-      }
-      
-      output <- normalApproxSF(subX1,subY, subZ, getinit$W,
+      output <- normalApproxSF(subX1,subY, subZ, W,
                                mdataM, mdataSM,
-                               pos.mode, presigma, pos.cov,
-                               H01, getinit$survtime, getinit$cmprsk,
-                               data$gamma1, data$alpha,
-                               CUH01,HAZ01, preSig, subdata$beta)
-      # }
+                               pos.mode,  sigma, pos.cov,
+                               H01, survtime, cmprsk,
+                               gamma1, alphaList,
+                               CUH01,HAZ01, Sig, beta)
       
-      # PAR UPDATE HERE - for debugging purposes
       beta <- output$beta
+      betaList <- output$betaList
       sigma <- output$sigmaVec
       Sig <- output$Sig
       
-      # tempsigma <- rbind(tempsigma, sigma)
-      # CHANGE THIS PART
       gamma1 <- output$phi1[1:ngamma]
-      alpha1 <- output$phi1[(ngamma+1):(length(output$phi1))] # 3 and 2 come from competing risk
+      alpha1 <- output$phi1[(ngamma+1):(length(output$phi1))]
       alpha1g <- vector("list", numBio)
       index = 0
       for(g in 1:numBio){
@@ -843,12 +621,11 @@ mvjmcs <- function(ydata, cdata, long.formula,
       
       alphaList <- list(alpha1g)
       
-      it <- it + 1
-      tempSig[[it]] <- Sig
       H01 <- output$H01
       
+      
       # leave condition
-      if((mvDiffSF(beta, prebeta, sigma, presigma, gamma1, pregamma1, 
+      if((mvDiffrelativeSF(beta, prebeta, sigma, presigma, gamma1, pregamma1, 
                    alpha1, prealpha1,
                    Sig, preSig, H01, preH01, tol) == 0) || (iter == maxiter) #|| (!is.list(GetEfun)) || (!is.list(GetMpara))
       ) {
@@ -860,10 +637,26 @@ mvjmcs <- function(ydata, cdata, long.formula,
     if (iter == maxiter) {
       writeLines("program stops because of nonconvergence")
       convergence = 0
-      sebeta <- sesigma <- segamma1 <- sealpha1 <- seSig <- NULL
+      sebeta <- sesigma <- segamma1 <- sealpha1 <- seSig <- vcov <- NULL
       
     } else {
       convergence = 1
+      
+      CUH01 <- rep(0, n)
+      HAZ01 <- rep(0, n)
+      CumuH01 <- cumsum(H01[, 3])
+      getHazardSF(CumuH01, survtime, cmprsk, H01, CUH01, HAZ01)
+      
+      data <- list(beta = betaList, gamma1 = gamma1,
+                   alpha = alphaList, sigma = sigma,
+                   Z = subZ, X1 = subX1, Y = subY, Sig = Sig,
+                   CUH01 = CUH01, HAZ01 = HAZ01,
+                   cmprsk = cmprsk, W = W)
+      
+      res <- future.apply::future_lapply(seq_len(numSubj), estepMV_workerSF, future.seed = TRUE, future.scheduling = 2, data, pREtotal)
+      pos.mode <- lapply(res, `[[`, "mode")
+      pos.cov  <- lapply(res, function(x) crossprod(x$ccov))
+      
       SEest <- getmvCovSF(beta, gamma1,
                           alpha1,  
                           H01, pos.cov, Sig, sigma, 
@@ -877,6 +670,8 @@ mvjmcs <- function(ydata, cdata, long.formula,
       sealpha1 <- SEest$sealpha1
       seSig <- SEest$seSig
       vcov <- SEest$vcov
+      FisherInfo <- SEest$FisherInfo
+      Score <- SEest$Score
     }
     
     end_time <- Sys.time()
@@ -900,7 +695,7 @@ mvjmcs <- function(ydata, cdata, long.formula,
     result <- list(beta = beta, betaList = betaList, gamma1 = gamma1, 
                    alpha1 = alpha1, H01 = H01, 
                    Sig = Sig, sigma = sigma, iter = iter, convergence = convergence, 
-                   vcov = vcov, sebeta = sebeta, segamma1 = segamma1,
+                   vcov = vcov, FisherInfo = FisherInfo, Score = Score, sebeta = sebeta, segamma1 = segamma1,
                    sealpha1 = sealpha1, seSig = seSig, sesigma = sesigma, 
                    pos.mode = pos.mode, pos.cov = pos.cov,
                    CompetingRisk = CompetingRisk, ydata = rawydata, cdata = rawcdata, 
