@@ -35,7 +35,7 @@ Rcpp::List getECpseudoSF(const Eigen::VectorXd & beta,
   
   Eigen::JacobiSVD<Eigen::MatrixXd> svd(Sig.inverse(), Eigen::ComputeThinU | Eigen::ComputeThinV);
   Eigen::VectorXd eigenSQ = svd.singularValues();
-  int i,j,q,t,db;
+  int i,j,q,t,db,u;
   for (i=0;i<eigenSQ.size();i++) {
     eigenSQ(i) = sqrt(eigenSQ(i));
   }
@@ -111,10 +111,12 @@ Rcpp::List getECpseudoSF(const Eigen::VectorXd & beta,
         FUNBS(i,j)+=exp(temp)*pow(bi(i),2);
       }
       if (p1a >= 2) {
+        u=0;
         for(i=1;i<p1a;i++)
         {
           for(t=0;t<p1a-i;t++) {
-            FUNBS(p1a+t+(i-1)*(p1a-1),j) += exp(temp)*bi(t)*bi(t+i);
+            FUNBS(p1a+u,j) += exp(temp)*bi(t)*bi(t+i);
+            u++;
           }
         }
       }
@@ -129,52 +131,16 @@ Rcpp::List getECpseudoSF(const Eigen::VectorXd & beta,
       }
 
       if (p1a >= 2) {
+        u=0;
         for(i=1;i<p1a;i++)
         {
           for(t=0;t<p1a-i;t++)
           {
-            FUNBSEC(p1a+t+(i-1)*(p1a-1),j)+=exp(temp)*exp(MultVV(alpha1,bi))*bi(t)*bi(t+i);
+            FUNBSEC(p1a+u,j)+=exp(temp)*exp(MultVV(alpha1,bi))*bi(t)*bi(t+i);
+            u++;
           }
         }
       }
-      
-      // FUNB.col(j)+=temp*bi;
-      // 
-      // for (i=0;i<p1a;i++) {
-      //   FUNBS(i,j)+=temp*pow(bi(i),2);
-      // }
-      // 
-      // if (p1a >= 2) {
-      //   for(i=1;i<p1a;i++)
-      //   {
-      //     for(t=0;t<p1a-i;t++) {
-      //       FUNBS(p1a+t+(i-1)*(p1a-1),j) += temp*bi(t)*bi(t+i);
-      //     }   
-      //   }
-      //   
-      // }
-      // 
-      // FUNEC(0,j)+=temp*exp(MultVV(alpha1,bi));
-      // 
-      // for (i=0;i<p1a;i++) {
-      //   FUNBEC(i,j)+=temp*bi(i)*exp(MultVV(alpha1,bi));
-      // }
-      // 
-      // for (i=0;i<p1a;i++) {
-      //   FUNBSEC(i,j)+=temp*exp(MultVV(alpha1,bi))*pow(bi(i),2);
-      // }
-      // 
-      // if (p1a >= 2) {
-      //   for(i=1;i<p1a;i++)
-      //   {
-      //     for(t=0;t<p1a-i;t++)
-      //     {
-      //       FUNBSEC(p1a+t+(i-1)*(p1a-1),j)+=temp*exp(MultVV(alpha1,bi))*bi(t)*bi(t+i);
-      //     }
-      //   }
-      // }
-      
-      
     }
     
     if(dem==0) {
