@@ -21,7 +21,7 @@ Rcpp::List getECstandardSF(const Eigen::VectorXd & beta,
   //calculate the square root of random effect covariance matrix 
   Eigen::JacobiSVD<Eigen::MatrixXd> svd(Sig, Eigen::ComputeThinU | Eigen::ComputeThinV);
   Eigen::VectorXd eigenSQ = svd.singularValues();
-  int i,j,q,t,db;
+  int i,j,q,t,db,u;
   for (i=0;i<eigenSQ.size();i++) {
     eigenSQ(i) = sqrt(eigenSQ(i));
   }
@@ -89,10 +89,12 @@ Rcpp::List getECstandardSF(const Eigen::VectorXd & beta,
       }
       
       if (p1a >= 2) {
+        u=0;
         for(i=1;i<p1a;i++)
         {
           for(t=0;t<p1a-i;t++) {
-            FUNBS(p1a+t+(i-1)*(p1a-1),j) += temp*bi(t)*bi(t+i);
+            FUNBS(p1a+u,j) += temp*bi(t)*bi(t+i);
+            u++;
           }   
         }
         
@@ -109,11 +111,13 @@ Rcpp::List getECstandardSF(const Eigen::VectorXd & beta,
       }
       
       if (p1a >= 2) {
+        u=0;
         for(i=1;i<p1a;i++)
         {
           for(t=0;t<p1a-i;t++)
           {
-            FUNBSEC(p1a+t+(i-1)*(p1a-1),j)+=temp*exp(MultVV(alpha1,bi))*bi(t)*bi(t+i);
+            FUNBSEC(p1a+u,j)+=temp*exp(MultVV(alpha1,bi))*bi(t)*bi(t+i);
+            u++;
           }
         }
       }
